/*
 * SPDX-FileCopyrightText: 2023 jordi fita mas <jfita@peritasoft.com>
 * SPDX-License-Identifier: AGPL-3.0-only
 */

package types

import (
	"context"
	gotemplate "html/template"
	"net/http"

	"golang.org/x/text/language"

	"dev.tandem.ws/tandem/camper/pkg/auth"
	"dev.tandem.ws/tandem/camper/pkg/carousel"
	"dev.tandem.ws/tandem/camper/pkg/database"
	httplib "dev.tandem.ws/tandem/camper/pkg/http"
	"dev.tandem.ws/tandem/camper/pkg/locale"
	"dev.tandem.ws/tandem/camper/pkg/template"
	"dev.tandem.ws/tandem/camper/pkg/uuid"
)

type PublicHandler struct {
}

func (h *PublicHandler) Handler(user *auth.User, company *auth.Company, conn *database.Conn) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		var head string
		head, r.URL.Path = httplib.ShiftPath(r.URL.Path)

		switch r.Method {
		case http.MethodGet:
			if !uuid.Valid(head) {
				http.NotFound(w, r)
				return
			}
			page, err := newPublicPage(r.Context(), conn, user.Locale, head)
			if database.ErrorIsNotFound(err) {
				http.NotFound(w, r)
				return
			} else if err != nil {
				panic(err)
			}
			page.MustRender(w, r, user, company, conn)
		default:
			httplib.MethodNotAllowed(w, r, http.MethodGet)
		}
	})
}

type publicPage struct {
	*template.PublicPage
	Name        string
	Carousel    []*carousel.Slide
	Prices      []*typePrice
	Features    []*typeFeature
	Spiel       gotemplate.HTML
	Info        gotemplate.HTML
	Facilities  gotemplate.HTML
	Description gotemplate.HTML
}

type typePrice struct {
	SeasonName    string
	SeasonColor   string
	MinNights     int
	PricePerNight string
	HasOptions    bool
}

type typeFeature struct {
	Icon string
	Name string
}

func newPublicPage(ctx context.Context, conn *database.Conn, loc *locale.Locale, slug string) (*publicPage, error) {
	prices, err := collectPrices(ctx, conn, loc.Language, slug)
	if err != nil {
		return nil, err
	}
	features, err := collectFeatures(ctx, conn, loc.Language, slug)
	if err != nil {
		return nil, err
	}
	page := &publicPage{
		PublicPage: template.NewPublicPage(),
		Carousel:   mustCollectSlides(ctx, conn, loc, slug),
		Prices:     prices,
		Features:   features,
	}
	row := conn.QueryRow(ctx, `
		select coalesce(i18n.name, campsite_type.name) as l10n_name
		     , coalesce(i18n.spiel, campsite_type.spiel)::text as l10n_description
		     , coalesce(i18n.info, campsite_type.info)::text as l10n_info
		     , coalesce(i18n.facilities, campsite_type.facilities)::text as l10n_facilities
		     , coalesce(i18n.description, campsite_type.description)::text as l10n_description
		from campsite_type
		    left join campsite_type_i18n as i18n on campsite_type.campsite_type_id = i18n.campsite_type_id and i18n.lang_tag = $1
		where slug = $2
		  and active
	`, loc.Language, slug)
	if err := row.Scan(&page.Name, &page.Spiel, &page.Info, &page.Facilities, &page.Description); err != nil {
		return nil, err
	}

	return page, nil
}

func collectPrices(ctx context.Context, conn *database.Conn, language language.Tag, slug string) ([]*typePrice, error) {
	rows, err := conn.Query(ctx, `
		select coalesce(i18n.name, season.name) as l10n_name
		     , to_color(season.color)::text
		     , coalesce(cost.min_nights, 1)
		     , to_price(coalesce(cost.cost_per_night, 0) + coalesce(option.cost_per_night, 0))
		     , option.cost_per_night is not null
        from season
		    left join season_i18n as i18n on season.season_id = i18n.season_id and i18n.lang_tag = $1
			left join (
			    campsite_type_cost as cost join campsite_type as type on cost.campsite_type_id = type.campsite_type_id and type.slug = $2
		    ) as cost on cost.season_id = season.season_id
        	left join (
        	    select season_id
        	         , sum(lower(range) * cost_per_night)::integer as cost_per_night
        	    from campsite_type_option
        	        join campsite_type using(campsite_type_id)
        			join campsite_type_option_cost using (campsite_type_option_id)
        	    where slug = $2
        	    group by season_id
   	        ) as option on option.season_id = season.season_id
		where season.active
    `, language, slug)
	if err != nil {
		return nil, err
	}

	var prices []*typePrice
	for rows.Next() {
		price := &typePrice{}
		if err := rows.Scan(&price.SeasonName, &price.SeasonColor, &price.MinNights, &price.PricePerNight, &price.HasOptions); err != nil {
			return nil, err
		}
		prices = append(prices, price)
	}
	return prices, nil
}

func collectFeatures(ctx context.Context, conn *database.Conn, language language.Tag, slug string) ([]*typeFeature, error) {
	rows, err := conn.Query(ctx, `
		select feature.icon_name
		     , coalesce(i18n.name, feature.name) as l10n_name
        from campsite_type_feature as feature
            join campsite_type using (campsite_type_id)
		    left join campsite_type_feature_i18n as i18n on feature.campsite_type_feature_id = i18n.campsite_type_feature_id and i18n.lang_tag = $1
		where campsite_type.slug = $2
    `, language, slug)
	if err != nil {
		return nil, err
	}

	var features []*typeFeature
	for rows.Next() {
		feature := &typeFeature{}
		if err := rows.Scan(&feature.Icon, &feature.Name); err != nil {
			return nil, err
		}
		features = append(features, feature)
	}
	return features, nil
}

func (p *publicPage) MustRender(w http.ResponseWriter, r *http.Request, user *auth.User, company *auth.Company, conn *database.Conn) {
	p.Setup(r, user, company, conn)
	template.MustRenderPublic(w, r, user, company, "campsite/type.gohtml", p)
}
